function [Elemente, Area, Abl, Kantenlaengen, Normalen, Kantennr, Kanten, ...
      Grid, NKV, NNV] = netz_info(Elemente, Koordinaten, ...
    Dirchlet, Neumann)
% Netzspezifische Information, die zur Zeitersparnis vorher bestimmt werden
global verbosity

lshift = [2,3,1];
AnzahlAussenkanten = size(Dirchlet,1)+size(Neumann,1);

% Umnummerierung, so dass die 1. Kante eines Elements die laengste Kante
% ist; (aH, April '98)
Kantenlaengen = reshape(sqrt(sum((Koordinaten(Elemente(:,[2,3,1]),:) - ...
    Koordinaten(Elemente,:)).^2,2)),size(Elemente));
[lm,off] = max(Kantenlaengen,[],2);
ind = sub2ind( size(Elemente), (1:size(Elemente,1))'*[1,1,1], ...
    rem(ones(size(Elemente,1),1)*(-1:1)+off*[1,1,1],3)+1);
Elemente = Elemente(ind);
Kantenlaengen = Kantenlaengen(ind);
clear llm off ind

% Area berechnen
Ecken = Koordinaten(reshape(Elemente',3*size(Elemente,1),1),:);
for j=1:2
  D(j:2:2*size(Elemente,1),:) = ...
      [Ecken(2:3:3*size(Elemente,1),j) - Ecken(1:3:3*size(Elemente,1),j),...
	Ecken(3:3:3*size(Elemente,1),j)  - Ecken(1:3:3*size(Elemente,1),j)];
end 
a = D(1:2:2*size(Elemente,1),1); b = D(1:2:2*size(Elemente,1),2);
c = D(2:2:2*size(Elemente,1),1); d = D(2:2:2*size(Elemente,1),2);
DetD = a.*d - b.*c;
if find(DetD<0) disp(['Warnung: negative Determinante einer ', ...
      'Transformationsmatrix']); end
Area = 1/2*DetD;

% Abl berechnen
Gradienten(:,1:3:3*size(Elemente,1)) = ([-d+c, b-a]./[DetD DetD])';
Gradienten(:,2:3:3*size(Elemente,1)) = ([d, -b]./[DetD DetD])';
Gradienten(:,3:3:3*size(Elemente,1)) = ([-c, a]./[DetD DetD])';
for j =1:3
  Abl(:,j:6:6*size(Elemente,1)) = [Gradienten(1,j:3:3*size(Elemente,1));
    zeros(1,size(Elemente,1));
    Gradienten(2,j:3:3*size(Elemente,1))/2];
  Abl(:,j+3:6:6*size(Elemente,1)) = [zeros(1,size(Elemente,1));
    Gradienten(2,j:3:3*size(Elemente,1));
    Gradienten(1,j:3:3*size(Elemente,1))/2];
end
clear a b c d DetD Gradienten

% fGaussPnts, die Auswertung der F'on f an den Gausspunkten jedes Elements,
% berechnen 
refGaussPunkte = [1/5 1/5;1/5 3/5;3/5 1/5;1/3 1/3];
for j=1:4
  GaussPnts(j:4:4*size(Elemente,1),:) = reshape((refGaussPunkte(j,:)*D' + ...
      reshape(Ecken(1:3:3*size(Elemente,1),:)',1,2*size(Elemente,1)))', ...
      size(Elemente,1),2);
end
%fGaussPnts = f(GaussPnts);
clear D

% Normalen berechnen
Kantenvektor = ...
    Koordinaten(reshape(Elemente(:,lshift)',3*size(Elemente,1),1),:) - ...
    Koordinaten(reshape(Elemente',3*size(Elemente,1),1),:);
Normalen = [Kantenvektor(:,2), ...
      -Kantenvektor(:,1)]./(sqrt(sum(Kantenvektor'.^2))'*[1 1]);
clear Kantenvektor
% 'Normalen' ist aufgelistet relativ zum Auftreten der zugehoerigen
% Kanten in Elemente. Normalen(3*j-(3-k),:) liefert die Normale zu der
% Kante des Elements 'Elemente(j,:)', die, im math. positiven Sinne, am
% Punkt 'Elemente(j,k)' ansetzt.
% (Sehr inkonsistent mit 'Kanten' weiter unten, ich weiss...)

% Kanten- (also auch Tangenten-) und Normalenvektoren der
% Neumann-Kantenstuecke 
NKV = Koordinaten(Neumann(:,2),:)- Koordinaten(Neumann(:,1),:);
NKV = NKV./[sqrt(sum(NKV.*NKV,2)),sqrt(sum(NKV.*NKV,2))]; % Neumann-Kanten
							  % Vektoren, normiert
NNV = [NKV(:,2),-NKV(:,1)];            % Neumann-Kanten Normalenvektoren

  
% Berechnung von Grid, der Matrix, die die Struktur der globalen STEMA
% wiedergibt. An den Korrelationspunkten enthaelt Grid den Wert realmin, sonst
% Null. realmin ist (zumindest auf der sun) so klein, das es bei der
% Assemblierung vernachlaessigt wird. (JA, aH, April '98)

eintraege = zeros(size(Elemente,1)*36,2);
[r,c]=meshgrid(1:3,1:3);
ind = [r(:);c(:)];
for i=1:size(Elemente,1)
  eintraege((i-1)*36+1:i*36-27,:) = reshape(2*Elemente(i,ind)-1,9,2);
  eintraege((i-1)*36+10:i*36-18,:) = reshape(2*Elemente(i,ind),9,2);
  eintraege((i-1)*36+19:i*36,1) = eintraege((i-1)*36+1:i*36-18,1);
  eintraege((i-1)*36+19:i*36,2) = eintraege([(i-1)*36+10:i*36-18, ...
	(i-1)*36+1:i*36-27],2);
end
eintraege = unique(eintraege,'rows');
Grid = sparse(eintraege(:,1),eintraege(:,2),realmin);
clear eintrage ind r c

% Folgendes liefert Kantennr=sparse(size(Koordinaten,1),size(Koordinaten,1))
% und full-Matrix Kanten mit size(Kanten) = AnzahlderKanten,4 und 
% zeilemweisen Eintraegen [Ecke1, Ecke2, Element1, Element2], wobei 
% Element2 = 0, falls Randkante und Ecke1, Ecke2 ist positiv orientiertes 
% Randstueck von Element1

Kantennr = sparse(size(Koordinaten,1),size(Koordinaten,1));
Kanten = zeros(ceil((size(Elemente,1)*3+AnzahlAussenkanten)/2),4);
cnt=0;
for j = 1:size(Elemente,1)
  for k = 1:3
    if ~Kantennr(Elemente(j,k),Elemente(j,lshift(k)))
      cnt=cnt+1;
      Kanten(cnt,:) = [Elemente(j,k), Elemente(j,lshift(k)), j, 0];
      Kantennr(Elemente(j,k),Elemente(j,lshift(k))) = cnt;
      Kantennr(Elemente(j,lshift(k)),Elemente(j,k)) = cnt;
    else
      Kanten(Kantennr(Elemente(j,k),Elemente(j,lshift(k))), 4) = j;
    end;
  end;
end;
Kanten = Kanten(find(Kanten(:,1)),:);


