function show_convergence(problem);

od=pwd;
cd ../CONVERGENCE_RESULTS
load(strcat('calculation_uniform_',problem));

x1=number_of_unknowns(1,:);
y1=norm_eta(1,:);
z1=norm_error(1,:);
x2=number_of_unknowns(2,:);
y2=norm_eta(2,:);
z2=norm_error(2,:);
x3=number_of_unknowns(3,:);
y3=norm_eta(3,:);
z3=norm_error(3,:);
if strcmp(problem,'ring_single')
   w=norm_error_exact;
end

load(strcat('calculation_adaptive_',problem));
cd(od)

xx1=number_of_unknowns(1,:);
yy1=norm_eta(1,:);
zz1=norm_error(1,:);
xx2=number_of_unknowns(2,:);
yy2=norm_eta(2,:);
zz2=norm_error(2,:);
xx3=number_of_unknowns(3,:);
yy3=norm_eta(3,:);
zz3=norm_error(3,:);

if strcmp(problem,'ring_single')
   ww=norm_error_exact;
end

for i=1:size(y3,2)
   ru(i)=y3(i)/z3(i);
end
disp('uniform mesh - eta/erorr factors');
disp(ru);
for i=1:size(yy3,2)
   ra(i)=yy3(i)/zz3(i);
end
disp('adaptive mesh - eta/erorr factors');
disp(ra);
meanra=mean(ra);
disp(meanra);

figure
fig1=gcf;
loglog(x1,z1,'--o',x2,z2,'-.o',x3,z3,'-o','markersize',10); %uniform error
hold on
loglog(xx1,zz1,'--d',xx2,zz2,'-.d',xx3,zz3,'-d','markersize',10); %adaptive error
xlabel('N');
ylabel('(estimated) error: ||C^{-1/2}(\Sigma - \Sigma_{ref})||_{L^{2}(\Omega)}');
legend('1 Newton step (uniform)','2 Newton steps (uniform)','3 Newton steps (uniform)',...
      '1 Newton step (adaptive)','2 Newton steps (adaptive)','3 Newton steps (adaptive)');
if strcmp(problem,'beam2D')
   dreieck2(10^(2.8),10^(-1.7),5,2,0.2);   %uniform
   dreieck2(10^(2.8),10^(-2.3),5,1,0.5);   %adaptive
   axis([1e1 10^(3.8) 10^(-2.5) 10^(-1.2)]);
end
if strcmp(problem,'ring')
   dreieck2(10^(2.8),10^(-0.7),5,2,0.5);   %uniform
   dreieck2(10^(2.8),10^(-1.4),5,1,0.5);   %adaptive
   axis([5 10^(3.8) 10^(-1.5) 10^(0.5)]);
end
if strcmp(problem,'ring_single')
   dreieck2(10^(2.8),10^(-0.8),5,2,0.5);   %uniform
   dreieck2(10^(2.8),10^(-1.5),5,1,0.5);   %adaptive
   axis([5 10^(3.8) 10^(-1.6) 10^(0.5)]);
end
if strcmp(problem,'Lshape')
   dreieck2(10^(2.5),10^(-2.1),5,2,0.3);   %uniform 
   dreieck2(10^(2.5),10^(-3.0),5,1,0.6);   %adaptive
   axis([8 10^(3.5) 10^(-3.2) 10^(-1.3)]);
end
if strcmp(problem,'cook')
   dreieck2(10^(2.8),10^(0.3),5,2,0.4);   %uniform 
   dreieck2(10^(2.8),10^(-0.4),5,1,0.5);   %adaptive
   axis([10^(1.3) 10^(3.8) 10^(-0.5) 10^(0.8)]);
end
if strcmp(problem,'platehole')
   dreieck2(10^(2.8),10^(1.7),5,2,0.3);   %uniform
   dreieck2(10^(2.8),10^(1.1),5,1,0.5);   %adaptive
   axis([10^(1.2) 10^(3.8) 10^(1) 10^(2.2)]);
end

cd FIGURES
print(gcf,'-depsc',strcat('error_',problem));
cd ..

figure(fig1+1)
loglog(x1,y1,'--o',x2,y2,'-.o',x3,y3,'-o','markersize',10); %uniform eta
hold on
loglog(xx1,yy1,'--d',xx2,yy2,'-.d',xx3,yy3,'-d','markersize',10); %adaptive eta
xlabel('N');
ylabel('ZZ-estimator: ||C^{-1/2}(\Sigma - A \Sigma)||_{L^{2}(\Omega)}');
legend('1 Newton step (uniform)','2 Newton steps (uniform)','3 Newton steps (uniform)',...
      '1 Newton step (adaptive)','2 Newton steps (adaptive)','3 Newton steps (adaptive)',0);
   
if strcmp(problem,'beam2D')
   dreieck2(10^(2.8),10^(-1.8),5,2,0.2);   %uniform 
   dreieck2(10^(2.8),10^(-2.7),5,1,0.5);   %adaptive
   axis([1e1 10^(3.8) 10^(-2.9) 10^(-1.2)]);
   %axis([1e1 10^(3.5) 10^(-2.3) 10^(-1)]);
end
if strcmp(problem,'ring')
   dreieck2(10^(2.8),10^(-1),5,2,0.5);   %uniform 
   dreieck2(10^(2.8),10^(-1.6),5,1,0.5);   %adaptive
   axis([5 10^(3.8) 10^(-1.8) 10^(0)]);
end
if strcmp(problem,'ring_single')
   dreieck2(10^(2.8),10^(-1),5,2,0.5);   %uniform 
   dreieck2(10^(2.8),10^(-1.6),5,1,0.5);   %adaptive
   axis([5 10^(3.8) 10^(-1.8) 10^(0)]);
end
if strcmp(problem,'Lshape')
   dreieck2(10^(2.5),10^(-2.3),5,2,0.3);   %uniform 
   dreieck2(10^(2.5),10^(-3.2),5,1,0.6);   %adaptive
   axis([8 10^(3.5) 10^(-3.4) 10^(-1.7)]);
end
if strcmp(problem,'cook')
   dreieck2(10^(2.8),10^(0.1),5,2,0.4);   %uniform 
   dreieck2(10^(2.8),10^(-0.5),5,1,0.5);   %adaptive
   axis([10^(1.3) 10^(3.8) 10^(-0.6) 10^(0.5)]);
end
if strcmp(problem,'platehole')
   dreieck2(10^(2.8),10^(1.5),5,2,0.3);   %uniform
   dreieck2(10^(2.8),10^(0.9),5,1,0.5);   %adaptive
   axis([10^(1.2) 10^(3.8) 10^(0.8) 10^(2.0)]);
end

cd FIGURES
print(gcf,'-depsc',strcat('eta_',problem));
cd ..

figure
if strcmp(problem,'ring_single')
   loglog(x3,z3,'-s',xx3,zz3,':s',x3,y3,'-ko',xx3,yy3,':o',x3,w,'-kv',xx3,ww,':v','markersize',10);
   ylabel('exact error, estimated error and ZZ-error estimator');
   legend('estimated error (uniform) ','estimated error (adaptive)',...
      'ZZ-error estimator (uniform)', 'ZZ error estimator (adaptive)',...
      'exact error (uniform)', 'exact error (adaptive)');
else
   loglog(x3,z3,'-o',xx3,zz3,'-d',x3,y3,'--o',xx3,yy3,'--d','markersize',10); 
 	ylabel('(estimated) error and ZZ-error estimator');
	xlabel('N');
   legend('(estimated) error (uniform) ','(estimated) error (adaptive)',...
      'ZZ-error estimator (uniform)', 'ZZ-error estimator (adaptive)');  
end
hold on

if strcmp(problem,'beam2D')
   dreieck2(10^(2.8),10^(-1.6),5,2,0.2);   %uniform 
   dreieck2(10^(2.8),10^(-2.6),5,1,0.5);   %adaptive
   axis([1e1 10^(3.8) 10^(-2.7) 10^(-1.2)]);
end
if strcmp(problem,'ring')
   dreieck2(10^(2.8),10^(-0.6),5,2,0.5);   %uniform 
   dreieck2(10^(2.8),10^(-1.7),5,1,0.5);   %adaptive
   axis([5 10^(3.8) 10^(-1.8) 10^(0.4)]);
end
if strcmp(problem,'ring_single')
   dreieck2(10^(2.8),10^(-0.7),5,2,0.5);   %uniform 
   dreieck2(10^(2.8),10^(-1.7),5,1,0.5);   %adaptive
   axis([5 10^(3.8) 10^(-1.8) 10^(0.4)]);
end
if strcmp(problem,'Lshape')
   dreieck2(10^(2.5),10^(-2.1),5,2,0.3);   %uniform 
   dreieck2(10^(2.5),10^(-3.2),5,1,0.6);   %adaptive
   axis([8 10^(3.5) 10^(-3.4) 10^(-1.2)]);
end
if strcmp(problem,'cook')
   dreieck2(10^(2.8),10^(0.3),5,2,0.4);   %uniform 
   dreieck2(10^(2.8),10^(-0.6),5,1,0.5);   %adaptive
   axis([10^(1.3) 10^(3.8) 10^(-0.8) 10^(0.8)]);
end
if strcmp(problem,'platehole')
   dreieck2(10^(2.8),10^(1.7),5,2,0.3);   %uniform
   dreieck2(10^(2.8),10^(0.9),5,1,0.5);   %adaptive
   axis([10^(1.2) 10^(3.8) 10^(0.8) 10^(2.2)]);
end

cd FIGURES
print(gcf,'-depsc',strcat('eta_and_error_',problem));
cd ..


