function eta_T = ZZ_Estimate(Elemente, Koordinaten, Dirichlet, ...
Neumann, NKV, NNV, Sigma,t)
% Fehlerschaetzer nach Zienkiewicz und Zhu
% eta = || G u_h - sigma(u_h) ||_T  fuer alle Elemente 

Sigma=[Sigma(:,1) Sigma(:,3) Sigma(:,3) Sigma(:,2)];

global lambda mu;
Flaeche = zeros(size(Elemente,1),1);
FlaecheOmega = zeros(size(Koordinaten,1),1);
MittelSigma = zeros(size(Koordinaten,1),4);
for j=1:size(Elemente,1)
  Flaeche(j) = det([1 1 1; Koordinaten(Elemente(j,:),:)'])/2;
  FlaecheOmega(Elemente(j,:))=FlaecheOmega(Elemente(j,:))+Flaeche(j);
  MittelSigma(Elemente(j,:),:) = MittelSigma(Elemente(j,:),:) + ...
      Flaeche(j) * [1; 1; 1] * Sigma(j,:);
end
MittelSigma = MittelSigma./FlaecheOmega(:,[1 1 1 1]);
if ~isempty(Neumann)             % evtl Neumannbedingung fuer G u_h erzwingen
  NeumannKnoten = unique(Neumann(:));
  for j=1:size(NeumannKnoten,1)
    [IN,tmp] = find(Neumann==NeumannKnoten(j)); % IN - Zeilen in Neumann
                                                % mit diesem Knoten
    [ID,tmp] = find(Dirichlet==NeumannKnoten(j)); % ID - Zeilen in Dirichlet 
    clear T gV;
    switch(int2str([size(IN,1),size(ID,1)]))
      case '1  1',                            % Neumann-Rand, Dirichlet-Rand
	T = [NNV(IN,:),0,0; 0,0,NNV(IN,:)];
	gV = g(Koordinaten(NeumannKnoten(j),:),NNV(IN,:),t)';
      case '1  2',                            % Gleitrand, Dirichlet-Rand
	T = [NKV(IN,1)*NNV(IN,:), NKV(IN,2)*NNV(IN,:)];
	gV = NKV(IN,:)*g(Koordinaten(NeumannKnoten(j),:),NNV(IN,:),t)';
      case '2  0',                            % Neumann-Rand, Neumann-Rand
	if abs(sum(NNV(IN(1),:).*NNV(IN(2),:)))<.99 % an einer Ecke
	  T = [NNV(IN,:), zeros(2,2); zeros(2,2),NNV(IN,:)];
	  gV([2 4],1) = g(Koordinaten(NeumannKnoten(j),:),NNV(IN(2),:),t)';
	  gV([1 3],1) = g(Koordinaten(NeumannKnoten(j),:),NNV(IN(1),:),t)';
	else                                 % auf einem Geradenstueck
	  T = [NNV(IN(1),:),0,0; 0,0,NNV(IN(1),:)];
	  gV = g(Koordinaten(NeumannKnoten(j),:),NNV(IN(1),:),t)';
	end
      case '2  1',                            % Neumann-Rand, Gleitrand
	% reinen Neumannrand in Neumann(IN,:) finden -> InotG
	[tmp,tmp2,InotG] = setxor(Dirichlet(ID,:),Neumann(IN,:),'rows');
	% IN_G - Gleitrand (Index relativ zu Neumann, nicht Neumann(IN,:))
	IN_G = setxor(IN,IN(InotG));
	if abs(sum(NNV(IN(1),:).*NNV(IN(2),:)))<.99 % an einer Ecke
	  T([2 3],:) = [NNV(IN(InotG),:),0,0; 0,0,NNV(IN(InotG),:)];
	  gV([2 3],1) = g(Koordinaten(NeumannKnoten(j),:),NNV(IN(InotG),:),t)';
	  T(1,:) = [NKV(IN_G,1)*NNV(IN_G,:), NKV(IN_G,2)*NNV(IN_G,:)];
	  gV(1,1) = NKV(IN_G,:)* g(Koordinaten(NeumannKnoten(j),:), ...
	      NNV(IN_G,:),t)';
	else                                 % auf einem Geradenstueck
	  T = [NNV(IN(InotG),:),0,0; 0,0,NNV(IN(InotG),:)];
	  gV = g(Koordinaten(NeumannKnoten(j),:),NNV(IN(InotG),:),t)';
	end
      case '2  2',                            % Gleitrand, Gleitrand
	if abs(sum(NNV(IN(1),:).*NNV(IN(2),:)))<.99 % an einer Ecke
	  T = [diag(NKV(IN,1))*NNV(IN,:), diag(NKV(IN,2))*NNV(IN,:)];
	  gV = diag(NKV(IN,:)*g(Koordinaten(NeumannKnoten([j j]),:), ...
	      NNV(IN,:))'); 
	else                                 % auf einem Geradenstueck
	  T = [NKV(IN(1),1)*NNV(IN(1),:), NKV(IN(1),2)*NNV(IN(1),:)];
	  gV = NKV(IN(1),:)*g(Koordinaten(NeumannKnoten(j),:),NNV(IN(1),:),t)';
	end
    end
    %T
    %gV
    tmp = [eye(4),T';T, zeros(size(T,1),size(T,1))]\ ...
	[MittelSigma(NeumannKnoten(j),:)'; gV];
    MittelSigma(NeumannKnoten(j),:) = tmp(1:4);
  end
end

c = [(2*mu+lambda)/(4*mu*(mu+lambda)),-lambda/(4*mu*(mu+lambda)),1/(2*mu)];
% inv. des Elast.-Moduls für plain strain
%(eps11, eps12, eps21, esp22)=Cinv(sigma11, sigma12, sigma21, sigma22)
Cinv = [c(1) 0 0 c(2); 0 c(3) 0 0; 0 0 c(3) 0; c(2) 0 0 c(1)]; 

eta_T = zeros(size(Elemente,1),1);
for j=1:size(Elemente,1)
    DiferSigma=MittelSigma(Elemente(j,:),:)-Sigma([j,j,j],:);
    eta_T(j) =Flaeche(j)*...
            sum(sum(DiferSigma.*([2,1,1;1,2,1;1,1,2]*DiferSigma*Cinv)))/12;
end
eta_T = sqrt(eta_T);

