function [P1local,P2local]=evaluate_P_local_multi(sigmaelasticlocal,P1prevlocal,P2prevlocal, ...
   mu_times_2, h1, h2, sigmay1, sigmay2, tolerance)

%A=sigma_elastic-(C+H)Pprev
A1=sigmaelasticlocal-((mu_times_2+h1)*P1prevlocal+mu_times_2*P2prevlocal);
A2=sigmaelasticlocal-(mu_times_2*P1prevlocal+(mu_times_2+h2)*P2prevlocal);
tr_over_2=(A1(1)+A1(2))/2; devA1=A1-[tr_over_2; tr_over_2; 0];
tr_over_2=(A2(1)+A2(2))/2; devA2=A2-[tr_over_2; tr_over_2; 0];
      
%%%%%% Iterative algorithm %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
P1=P1prevlocal; 
P2=P2prevlocal;
P1old=P1; 
P2old=P2;
normoldsquare=P1old(1)^2+P1old(3)^2+P2old(1)^2+P2old(3)^2;

%elem1=[]; elem2=[]; % elem1 and elem2 only for convergence testing 

%initialization of Romberg extrapolation 
k=0; i=0; y=zeros(3);

while 1
    %minimization in direction P1
    devA=devA1-mu_times_2*P2;
    sigmay=sigmay1; h=h1;  
    normdevA=sqrt(2*(devA(1)^2+devA(3)^2));
    if (normdevA<sigmay)  
        normP=0;
    else
        normP=(normdevA-sigmay)/normdevA/(mu_times_2+h);
    end
    P1=devA*normP;
    
    %minimization in direction P2
    devA=devA2-mu_times_2*P1; 
    sigmay=sigmay2; h=h2;
    normdevA=sqrt(2*(devA(1)^2+devA(3)^2));
    if (normdevA<sigmay)  
        normP=0;
    else
        normP=(normdevA-sigmay)/normdevA/(mu_times_2+h);
    end
    P2=devA*normP;  
    
    %elem1=[elem1 P1(1,1)]; elem2=[elem2 P2(1,1)]; %elem1 and elem2 only for convergence testing 
    
    %Romberg extrapolation of P2
    i=i+1; 
    y(:,i)=P2;
    if (i==3)
        k=k+1;
        p12=zeros(3,1);
        for j=1:3
            x1=y(j,2)-y(j,1); x2=y(j,3)-y(j,2);
            p1=y(j,1); p2=y(j,2);
            if (x1==x2)
                p12(j)=0;
            else
                p12(j)=(-x1*p2+x2*p1)/(x2-x1);
            end
        end  
        i=0;
        P2=p12; %this line activates Romberg acceleration
        
        normoldsquare=P1(1)^2+P1(3)^2+P2(1)^2+P2(3)^2;   
    else
        normnewsquare=P1(1)^2+P1(3)^2+P2(1)^2+P2(3)^2;   
        differencesquare= (P1(1)-P1old(1))^2 + (P1(3)-P1old(3))^2 ...
                         +(P2(1)-P2old(1))^2 + (P2(3)-P2old(3))^2;
        if differencesquare>0
           differencesquare=differencesquare/(normnewsquare+normoldsquare);
           if differencesquare<tolerance^2
                break
           end 
           normoldsquare=normnewsquare;
        else 
            break
        end 
    end
    
    P1old=P1; 
    P2old=P2;
end

%%%% End of Iterative algorithm %%%%%%%%%%%%%%%%%%%%%%
    
P1local=P1prevlocal+P1;
P2local=P2prevlocal+P2;

%save zigzag elem1 elem2
