function [U,U_coarse,P1,P1_coarse,P2,P2_coarse,iterations]=...
nested_iteration_refinements(P1prev,P2prev,U,time,Newtonsteps,refinements)
global Koordinaten Elemente Neumann Dirichlet %mesh invariants
global gv fv
global problem
global counter ErrorEstimator N 
global Sigma
global Kantennr
global refinement_technique error_indicator
global t
global pictures

iterations=[];
for i=0:refinements 
    %refinement start --------------------------------
    if i>0
        % generate mesh refinement 
        switch error_indicator
        case {'ZZ', 'residual','both'}                  
            switch refinement_technique
                case 'AAlg7.1'
                    VK=AAlg_71(eta,Kantennr,KantennrzuElemente);
                case 'AAlg7.2'
                    VK=AAlg_72(eta,Kantennr,KantennrzuElemente);  
            end
        case 'interface'  
            [VK,alphas]=AAlg_interface(yield_linear,Kantennr);   
        end
        VK = BlauGruen(Elemente,Koordinaten,Kantennr,VK,ElementzuKante);
        numberRefinedEdges=nRefinedEdges(Elemente,Kantennr,VK,KantennrzuElemente);
        Prolongation_linear = GeneriereProlongation(Kantennr,VK);
        %Prolongation U
        U=Prolongation_linear*U;  
 
        [Koordinaten,Elemente,Dirichlet,Neumann]=Verfeinerung(Koordinaten,...
               Elemente,Dirichlet,Neumann,Kantennr,VK);
        %Prolongation P 
        if size(t,2)==1
            P1prev=zeros(size(Elemente));
            P2prev=P1prev;
        else
            P1prev=prolongate_constant(P1prev,numberRefinedEdges);
            P2prev=prolongate_constant(P2prev,numberRefinedEdges);        
        end          
                                                            
        switch problem 
        case 'ring',
            Koordinaten=correct_ring_coordinates(Koordinaten,Neumann);
        case 'platehole'
            Koordinaten=correct_platehole_coordinates(Koordinaten,Neumann);
        end; 
        
        fprintf('\nmesh refined \n');   
    end
    %refinement end -----------------------------------------
    
    [Kantennr,Elemente,KantennrzuElemente,ElementzuKante]=GeneriereKantennr(Elemente,Koordinaten);   
    
    fprintf('number of elements: %d \n',size(Elemente,1));
    N(i+1,counter)=2*size(Koordinaten,1);
    fprintf('number of unknowns: %d \n',N(i+1,counter));
  
    mesh_preparation; %gets mesh ready - STEMAelastic + maske and others
    incorporate_Dirichlet;
    if strcmp(problem,'Lshape')
        for j=1:size(Dirichlet,1)
            U(Dirichlet(j,1),:)=Lshape_Dirichlet(Koordinaten(Dirichlet(j,1),:));
            U(Dirichlet(j,2),:)=Lshape_Dirichlet(Koordinaten(Dirichlet(j,2),:));
        end
    end
    fv=evaluate_fv(time,Koordinaten,Elemente);
    gv=evaluate_gv(time,Koordinaten,Neumann);
  
    [U,P1,P2,iterations_Newton]=FEM_Newton(P1prev,P2prev,U,Newtonsteps);
    if i==0
        U_coarse=U;
        P1_coarse=P1;
        P2_coarse=P2;
    end
    iterations=[iterations;iterations_Newton];
     
    if ~isempty(Neumann)
        NKV = Koordinaten(Neumann(:,2),:)- Koordinaten(Neumann(:,1),:);
        NKV = NKV./[sqrt(sum(NKV.*NKV,2)),sqrt(sum(NKV.*NKV,2))]; 
        % Neumann-Kanten % Vektoren, normiert
        NNV = [NKV(:,2),-NKV(:,1)]; % Neumann-Kanten Normalenvektoren
    else NKV=[]; NNV=[];
    end
   
    %[Kantennr,Elemente]=GeneriereKantennr(Elemente,Koordinaten);
  
  
    Sigma=evaluate_sigma(U,P1,P2);
    [yield]=evaluate_yield(U,0*P1,0*P2);
    for k=1:size(yield,2)
        if (yield(k)>0) 
             yield(k)=1;
        else yield(k)=-1;
        end
    end  
    [yield_linear]=evaluate_average(yield);
  
    switch error_indicator
        case 'both' 
            eta=ZZ_Estimate(Elemente, Koordinaten, Dirichlet,...
                                Neumann, NKV, NNV, Sigma,time);
            fprintf('ZZ-error estimator %f \n',norm(eta));                                                        
            eta=RESIDUAL_Estimate(Koordinaten,Sigma, Neumann,...
                                       time,Kantennr,ElementzuKante);
            fprintf('Residual-error estimator %f \n',norm(eta));
        case 'ZZ'      
            eta=ZZ_Estimate(Elemente, Koordinaten, Dirichlet,...
                                Neumann, NKV, NNV, Sigma,time);   
            fprintf('ZZ-error estimator %f \n',norm(eta));
        case 'residual'      
            eta=RESIDUAL_Estimate(Koordinaten,Sigma,Neumann,...
                                       time,Kantennr,ElementzuKante);
            fprintf('Residual-error estimator %f \n',norm(eta));
    end
    ErrorEstimator(i+1,counter)=norm(eta);
  
  save convergence N ErrorEstimator 
  if strcmp(pictures,'yes') display_deformation
  end
end


