%Author: Jan Valdman, Jan.Valdman@sfb013.uni-linz.ac.at
%Version: 14.10.2005

clear all
declaration_of_variables % specifies global variables

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Newtonsteps='3'; %number of Newtonsteps (as a string), 'three_stages' or 'fixed_residual'
error_indicator='residual'; %'ZZ' or 'residual' or 'interface'
refinement_technique='AAlg7.2'; %'AAlg7.1' or 'AAlg7.2'
theta=0.5; 
plasticstiffnessmatrix='no'; % 'yes' (much faster performance) or 'no' 
pictures='yes'; % 'yes' (for pictures displayed) or 'no';
refinements=8; %number of refinements 
maximum_Newton_step=100;
step_min=0; step_max=100; %time adaptivity
tolerance=1e-8; %for the plastic-dependence scheme
tolerance_Newton=10e-6;
maximum_Newton_step=100;
yield_type='multi'; % 'multi' or 'single' or 'elastic';
problem='Lshape_new'; %beam2D', 'beam1D' or 'ring' or 'Lshape' or 'Lshape_new' or 'cook' or 'platehole';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Assigning problems to their directories
switch problem
case {'beam1D','beam2D'}
   problem_directory='beam_mesh';
   %problem_directory='beam2_mesh'
   %problem_directory='beam_mesh_smallest'
case 'ring',
   problem_directory='ring_mesh';
case 'Lshape'
   problem_directory='Lshape_mesh';
case 'Lshape_new'
   problem_directory='Lshape_mesh_new';  
case 'cook'
   problem_directory='cook_mesh';
case 'platehole'
   problem_directory='platehole_mesh';
end;

last_uniform_refinements=1;  %minimal value=1, for the generation of the reference solution

%material and time properties
wd=cd;
directory=strcat('../',problem_directory); cd (directory);
problem_properties;
cd (wd);

%initialize coarse mesh
[Koordinaten,Elemente,Dirichlet,Neumann]=load_mesh(problem_directory,0); 

%global elasticity matrix C 
mu_times_2=mu*2;
C=mu*[2 0 0;0 2 0;0 0 1] + lambda*[1 1 0;1 1 0;0 0 0];

%uniform refinement of the coarse mesh if the mesh is too small 
while size(Elemente,1)<1
  [Koordinaten,Elemente,Dirichlet,Neumann]=Rotverfeinerung(Koordinaten,...
      Elemente,Dirichlet,Neumann);
  switch problem 
  case 'ring',
        Koordinaten=correct_ring_coordinates(Koordinaten,Neumann);
  case 'platehole'
        Koordinaten=correct_platehole_coordinates(Koordinaten,Neumann);
  end;  
end

%saving coarse mesh variables
Koordinaten_coarse=Koordinaten;
Elemente_coarse=Elemente;
Dirichlet_coarse=Dirichlet;
Neumann_coarse=Neumann;

%initial conditions set to zero
U_coarse=zeros(size(Koordinaten,1),2); %defined on every node
P1_coarse=zeros(size(Elemente,1),3); %defined on every element - constant
P2_coarse=zeros(size(Elemente,1),3); %defined on every element - constant

%seting up state variables
state=[]; %0 -elastic, 1-first plastic, 2-second plastic
ZZ=zeros(1,1); N=zeros(1,1); %values of ZZ estimator and the number of unknowns
numberoftimes=size(t,2); %number of time point
counter=1; %time counter

%hysteresis_u=[]; hysteresis_g=[]; %for hysteresis modeling
%condestA=[];

%cycle over all discrete times
further=1;
while further
    
   %switching to coarse mesh
   Koordinaten=Koordinaten_coarse;
   Elemente=Elemente_coarse;
   Dirichlet=Dirichlet_coarse;
   Neumann=Neumann_coarse;
         
   %testing the convergence on the coarse mesh for the time adaptivity 
   %disp('Running FEM + Newton method on the coarse mesh');
   %[U,U_coarse,P1,P1_coarse,P2,P2_coarse,iterations_coarse]=...
   %nested_iteration_refinements(P1prev,...
   %   P2prev,Uprev,t(counter),'fixed_residual',0);

   Uprev_coarse=U_coarse;
   P1prev_coarse=P1_coarse;
   P2prev_coarse=P2_coarse;
   
   %improving the space error - mesh-refining
   [U,U_coarse,P1,P1_coarse,P2,P2_coarse,iterations_fine]=...
      nested_iteration_refinements(P1prev_coarse,...
      P2prev_coarse,Uprev_coarse,t(counter),Newtonsteps,refinements);
  
   iterations(:,counter)=iterations_fine;
   iterations_coarse=iterations_fine(1);
   
   %which state - elastic, single-yield or two-yield?
   if (norm(P1)+norm(P2)==0)
         fprintf('elastic behaviour \n\n');
         state=[state 0];
   else
      if (norm(P2)>0)
         fprintf('two-yield plasticity behaviour \n\n');
         state=[state 2];       
      else fprintf('single-yield plasticity behaviour \n\n');
           state=[state 1];
      end
   end
   
   %output for hysteresis behavior
   %generate_hysteresis;
   %figure(30);
   %plot(hysteresis_u,hysteresis_g,'x-');
   %M(counter)=getframe;
   
   %adaptive time-stepping
   if (counter==numberoftimes)
      further=0;
   else
      if iterations_coarse>step_max
         t=adaptive_time(t,counter+1,0.5);
      else
         if iterations_coarse<step_min
            t=adaptive_time(t,counter+1,2);
         else
            t=adaptive_time(t,counter+1,1);
         end
      end
      
      numberoftimes=size(t,2);
      counter=counter+1;
   end
end

%save
%movie(M);
%movie2avi(M,'plasticity.avi','FPS',2);
