/************************
 * Test run the library *
 ************************/

#include <iostream>
#include <time.h>
#include "solve_sys.H"
using namespace std;

void oneStep(long double *devA, long double b, long double denominator, long double *outputP){ 
  long double normdevA=sqrt(devA[0]*devA[0]+devA[1]*devA[1]+devA[2]*devA[2]+devA[3]*devA[3]);
  //cout << "Normdev:" << normdevA << endl;
  
  if (normdevA <= b) {outputP[0]=0; outputP[1]=0; outputP[2]=0; outputP[3]=0;}
  else {
  	outputP[0]=((normdevA-b)/normdevA/denominator)*devA[0];
	outputP[1]=((normdevA-b)/normdevA/denominator)*devA[1];
	outputP[2]=((normdevA-b)/normdevA/denominator)*devA[2];
	outputP[3]=((normdevA-b)/normdevA/denominator)*devA[3];
	};
  }
  
void dev(long double *A, long double *outputdevA){
 	long double trace;
	//Implement for 2x2 matrix only!
	trace=A[0]+A[3];
  	outputdevA[0]=A[0]-trace/2;
  	outputdevA[1]=A[1];
  	outputdevA[2]=A[2]-trace/2;
  	outputdevA[3]=A[3];
	}

//multiply mu*P[] and store in Matrix[]
void multiply(long double mu, long double *P, long double *Matrix){
	Matrix[0]=mu*P[0];
	Matrix[1]=mu*P[1];
	Matrix[2]=mu*P[2];
	Matrix[3]=mu*P[3];
} 



void shift_nomu(long double *devA, long double *P, long double *shiftedMatrix){
	shiftedMatrix[0]=devA[0]-P[0];
	shiftedMatrix[1]=devA[1]-P[1];
	shiftedMatrix[2]=devA[2]-P[2];
	shiftedMatrix[3]=devA[3]-P[3];
} 

void shift(long double *devA, long double mu, long double *P, long double *shiftedMatrix){
	shiftedMatrix[0]=devA[0]-2*mu*P[0];
	shiftedMatrix[1]=devA[1]-2*mu*P[1];
	shiftedMatrix[2]=devA[2]-2*mu*P[2];
	shiftedMatrix[3]=devA[3]-2*mu*P[3];
} 

long double FrobeniusNorm(long double *A){
	return (sqrt(A[0]*A[0]+A[1]*A[1]+A[2]*A[2]+A[3]*A[3]));
}

long double FrobeniusNorm2(long double *A){
	return (A[0]*A[0]+A[1]*A[1]+A[2]*A[2]+A[3]*A[3]);
}



// example values (example 6.1 from thesis)
long double sigma1 = 1.0;
long double sigma2 = 2.0;
long double h1 = 1.0;
long double h2 = 1.0;
long double mu = 1.0;

long double  A1[4]={20,0,0,0}, A2[4]={20,0,0,0}; 
long double d11=200.0; //d11 = 2; //((A1[1]*A1[1]+A1[4]*A1[4]))/2.0-A1[1]*A1[4]+A1[2]*A1[2]+A1[3]*A1[3]; //2.0;
long double d22 = 200.0; //(A2[1]*A2[1]+A2[4]*A2[4])/2-A2[1]*A2[4]+A2[2]*A2[2]+A2[3]*A2[3]; //2.0;
long double d12 = 200.0; //(A1[1]*A2[1]+A1[4]*A2[4]-A1[1]*A2[4]-A1[4]*A2[1])/2+A1[2]*A2[2]+A1[3]*A2[3];

/*
  The main routine (example):
*/

int main (int argc, char **argv) {

  long double xi1, xi2;
  clock_t start, end;
  double diff;
  
  
  //////////start of numerical calculation of Jan Valdman ///////////////////////
  long double P1[4], P2[4];
  long double trace;
  long double devA1[4], devA2[4];
  long double denominator1=2*mu+h1;
  long double denominator2=2*mu+h2; 
  long double shiftedMatrix[4];
  int cycles=1000;
  int iterations=30;
  
  //Initialization devA1, devA2
  
  
  dev(A1,devA1);
  dev(A2,devA2);

  start = clock();
  for (long int j=1; j<cycles; j++) 
  {
      P1[0]=P1[1]=P1[2]=P1[3]=0;
      P2[0]=P2[1]=P2[2]=P2[3]=0;
      for (int i=1; i<iterations; i++) 
	{
	//P2 direction step
	shift(devA2,mu,P1,shiftedMatrix);
	oneStep(shiftedMatrix,sigma2,denominator2,P2);  
	//P1 direction step
	shift(devA1,mu,P2,shiftedMatrix);
	oneStep(shiftedMatrix,sigma1,denominator1,P1);
	}
  }
  end = clock();
  diff = (double)(end - start)/(double)CLOCKS_PER_SEC; 

  cout << "P1: " << P1[0] << " " << P1[1] << endl;
  cout << "P2: " << P2[0] << " " << P2[1] << endl;
  cout << "xi1: " <<  FrobeniusNorm(P1) << endl;
  cout << "xi2: " <<  FrobeniusNorm(P2) << endl;
  cout << "Total Time  : " << diff << endl;
  cout << "Time/Cycles : " << diff/cycles << endl;
  cout << "----------" << endl;

  start = clock();
  long double normdevA1_2=(FrobeniusNorm2(devA1));
  long double normdevA2_2=(FrobeniusNorm2(devA2));
  long double devA1devA2 = devA1[0]*devA2[0]+devA1[1]*devA2[1]+devA1[2]*devA2[2]+devA1[3]*devA2[3];
  long double P1devA1 = 0;
  long double P1devA2 = 0;
  long double P2devA1 = 0;
  long double P2devA2 = 0;
  long double normP1 = 0;
  long double normP2 = 0;
  long double normP1_2 = 0;
  long double normP2_2 = 0;
  long double help;
  denominator2=1+h2/(2*mu); //2*mu+h2;
  denominator1=1+h1/(2*mu); //2*mu+h2;
  for (long int j=1; j<cycles; j++) 
    {
	
	P1[0]=P1[1]=P1[2]=P1[3]=0;
  	P2[0]=P2[1]=P2[2]=P2[3]=0;

  for (int i=1; i<iterations; i++) 
  {
    //P2 direction step
    shift_nomu(devA2,P1,shiftedMatrix);
    //    cout << shiftedMatrix[0] << "shM" << endl;
    //    cout << denominator2 << endl;
    help = sqrt(normdevA2_2 - P1devA2 + normP1_2);
    if(help - sigma2 < 0){
      normP2 = P2devA1 = P2devA2 = 0;
      }
    else
      {
	normP2 = (help - sigma2)/denominator2;
	help = normP2/help;	
	multiply(help,shiftedMatrix,P2);
	P2devA1 = help*(devA1devA2 - P1devA1);
	P2devA2 = help*(normdevA2_2 - P1devA2);
      }

    
    //    oneStep(shiftedMatrix,sigma2,denominator,P2);  
    //P1 direction step
    shift_nomu(devA1,P2,shiftedMatrix);
    help = sqrt(normdevA1_2 - P2devA1 + normP2_2);
    if(help - sigma1 < 0){
      normP1 = P1devA1 = P1devA2 = 0;
      }
    else
      {
    normP1 = (help - sigma1)/denominator1;
    help = normP1/help;
    multiply(help,shiftedMatrix,P1);
    P1devA1 = help*(normdevA1_2 - P2devA1);
    P1devA2 = help*(devA1devA2 - P2devA2);
      }
    //    shift(devA1,mu,P2,shiftedMatrix);
    //denominator=2*mu+h1;
    //oneStep(shiftedMatrix,sigma1,denominator,P1);
  }
    }
//scale back
 multiply(1/(2*mu),P1,P1); 
 multiply(1/(2*mu),P2,P2);
 
  end = clock();
  diff = (double)(end - start)/(double)CLOCKS_PER_SEC; 

  cout << "P1: " << P1[0] << " " << P1[1] << endl;
  cout << "P2: " << P2[0] << " " << P2[1] << endl;
  cout << "xi1: " <<  FrobeniusNorm(P1) << endl;
  cout << "xi2: " <<  FrobeniusNorm(P2) << endl;
  cout << "Total Time  : " << diff << endl;
  cout << "Time/Cycles : " << diff/cycles << endl;
  cout << "----------" << endl;

  //////end of numerical calculation of Jan Valdman //////////////////////////////////////////////
  
  
  init();
  start = clock();
  solve(sigma1, sigma2, h1, h2, mu, d11, d22, d12, xi1, xi2);
  end = clock();
  diff = (double)(end - start)/(double)CLOCKS_PER_SEC;

  cout << "xi1: " <<  xi1 << endl;
  cout << "xi2: " <<  xi2 << endl;
  cout << "Time/Cycles : " << diff << endl;
  cout << "----------" << endl;








}
